
# q-expansion of newform 8010.2.a.l, downloaded from the LMFDB on 14 May 2024.

# We generate the q-expansion using the Hecke eigenvalues a_p at the primes.
# Each a_p is given as a linear combination
# of the following basis for the coefficient ring.

def make_data():

    from sage.all import prod, floor, prime_powers, gcd, QQ, primes_first_n, next_prime, RR

    def discrete_log(elts, gens, mod):
        # algorithm 2.2, page 16 of https://arxiv.org/abs/0903.2785
        def table_gens(gens, mod):
            T = [1]
            n = len(gens)
            r = [None]*n
            s = [None]*n
            for i in range(n):
                beta = gens[i]
                r[i] = 1
                N = len(T)
                while beta not in T:
                    for Tj in T[:N]:
                        T.append((beta*Tj) % mod)
                    beta = (beta*gens[i]) % mod
                    r[i] += 1
                s[i] = T.index(beta)
            return T, r, s
        T, r, s = table_gens(gens, mod)
        n = len(gens)
        N = [ prod(r[:j]) for j in range(n) ]
        Z = lambda s: [ (floor(s/N[j]) % r[j]) for j in range(n)]
        return [Z(T.index(elt % mod)) for elt in elts]
    def extend_multiplicatively(an):
        for pp in prime_powers(len(an)-1):
            for k in range(1, (len(an) - 1)//pp + 1):
                if gcd(k, pp) == 1:
                    an[pp*k] = an[pp]*an[k]
    from sage.all import PolynomialRing, NumberField
    R = PolynomialRing(QQ, "x")
    f = R(poly_data)
    K = NumberField(f, "a")
    betas = [K.gens()[0]**i for i in range(len(poly_data))]
    convert_elt_to_field = lambda elt: sum(c*beta for c, beta in zip(elt, betas))
    # convert aps to K elements
    primes = primes_first_n(len(aps_data))
    good_primes = [p for p in primes if not p.divides(level)]
    aps = map(convert_elt_to_field, aps_data)
    if not hecke_ring_character_values:
        # trivial character
        char_values = dict(zip(good_primes, [1]*len(good_primes)))
    else:
        gens = [elt[0] for elt in hecke_ring_character_values]
        gens_values = [convert_elt_to_field(elt[1]) for elt in hecke_ring_character_values]
        char_values = dict([(
            p,prod(g**k for g, k in zip(gens_values, elt)))
            for p, elt in zip(good_primes, discrete_log(good_primes, gens, level))
            ])
    an_list_bound = next_prime(primes[-1])
    an = [0]*an_list_bound
    an[1] = 1
    
    from sage.all import PowerSeriesRing
    PS = PowerSeriesRing(K, "q")
    for p, ap in zip(primes, aps):
        if p.divides(level):
            euler_factor = [1, -ap]
        else:
            euler_factor = [1, -ap, p**(weight - 1) * char_values[p]]
        k = RR(an_list_bound).log(p).floor() + 1
        foo = (1/PS(euler_factor)).padded_list(k)
        for i in range(1, k):
            an[p**i] = foo[i]
    extend_multiplicatively(an)
    return PS(an)
level = 8010
weight = 2
poly_data = [0, 1]

# The basis for the coefficient ring is just the power basis
# in the root of the defining polynomial above.
hecke_ring_character_values = None
aps_data = [[1], [0], [1], [0], [0], [0], [-2], [-6], [-4], [-2], [-2], [-8], [2], [8], [-8], [-6], [-6], [-6], [4], [12], [-6], [-8], [0], [1], [-2], [-6], [8], [-12], [10], [20], [-20], [0], [12], [0], [10], [-14], [-2], [4], [8], [-18], [16], [-2], [6], [-12], [16], [0], [14], [8], [4], [-14], [-14], [26], [-2], [16], [2], [32], [-18], [-20], [-30], [10], [20], [-4], [12], [-4], [-4], [-6], [-28], [-8], [12], [-6], [16], [-18], [-24], [2], [14], [-36], [18], [-28], [2], [14], [-26], [2], [-34], [-12], [26], [-20], [-2], [-8], [18], [8], [36], [8], [16], [-34], [-34], [-12], [-6], [-18], [28], [-2], [32], [-24], [-28], [30], [-30], [-16], [12], [4], [-38], [6], [8], [46], [36], [36], [20], [-10], [20], [-28], [0], [16], [-22], [22], [-12], [8], [8], [2], [10], [6], [0], [30], [-10], [36], [52], [-22], [-14], [10], [-24], [-28], [10], [-6], [4], [22], [40], [24], [-34], [26], [0], [-48], [-34], [-8], [-8], [50], [-16], [-12], [-28], [-52], [-14], [-34], [-2], [18], [20], [36], [-20], [-36], [-2], [24], [38], [-22], [-34], [-30], [-12], [-2], [-38], [40], [-34], [-54], [0], [18], [16], [50], [16], [50], [-14], [60], [-4], [54], [-2], [-4], [10], [46], [54], [36], [26], [54], [-8], [30], [-66], [-4], [-12], [-12], [-46], [-30], [-22], [-10], [-30], [-8], [-30], [52], [-30], [-20], [16], [22], [-48], [-20], [56], [-42], [-40], [-18], [48], [12], [50], [-4], [-38], [56], [-28], [26], [50], [-18], [4], [-6], [4], [14], [44], [-54], [-40], [24], [-46], [-42], [26], [28], [52], [36], [64], [2], [-42], [-6], [52], [-6], [-58], [16], [-30], [-50], [-8], [-62], [78], [60], [30], [-4], [-36], [2], [8], [-20], [58], [26], [56], [-20], [14], [30], [-36], [66], [62], [40], [-40], [32], [-32], [72], [32], [22], [6], [-38], [-16], [10], [0], [-26], [76], [8], [38], [-6], [-48], [-34], [-54], [-32], [-30], [-18], [-74], [-30], [26], [20], [-12], [12], [40], [-74], [54], [36], [52], [72], [28], [66], [-32], [-64], [48], [-6], [34], [20], [-40], [38], [-58], [12], [-72], [-26], [16], [-26], [2], [36], [42], [30], [-14], [4], [8], [52], [6], [-92], [-10], [-4], [50], [12], [-38], [-28], [-50], [-16], [-2], [-14], [-38], [-26], [-24], [36], [-78], [8], [-18], [-12], [52], [52], [-30], [-24], [-22], [-54], [-84], [8], [-16], [0], [38], [6], [72], [-38], [-92], [44], [-60], [-96], [-2], [12], [0], [24], [-50], [-62], [56], [-48], [32], [0], [-34], [-50], [18], [-72], [-60], [-72], [-76], [-80], [32], [-12], [-28], [24], [-6], [-60], [86], [72], [-46], [-64], [-44], [90], [-70], [-54], [-50], [84], [40], [46], [-78], [12], [-28], [-30], [76], [-78], [88], [-6], [36], [-74], [50], [-70], [-30], [-8], [54], [-56], [10], [-94], [-48], [80], [-38], [-58], [104], [-6], [44], [-90], [-58], [-2], [-62], [72], [-82], [58], [-74], [-46], [72], [40], [-28], [42], [-58], [-76], [-70], [-34], [24], [8], [14], [-54], [52], [-58], [-4], [-6], [68], [6], [74]]
